/* 
 * Crystal Mud Client
 * Copyright (C) 1998-2002 Abigail Brady
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 */

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>
#include <errno.h>
#include <fcntl.h>
#include <netdb.h>
#include <resolv.h>

#include "config.h"

using namespace std;

#include "Socket.h"

class InAddr4 : public InAddr {
  struct sockaddr_in s;
public:
  InAddr4(struct in_addr *si) {
    s.sin_family = AF_INET;
    s.sin_port = 0;
    s.sin_addr = *si;
  }
  virtual ~InAddr4() { };
  void set_port(int port) { s.sin_port = htons(port); };
  virtual string tostring() {
    return inet_ntoa(s.sin_addr);
  }
  sockaddr *addr() {
    return (sockaddr*)&s;
  }
  socklen_t addr_len() {
    return sizeof s;
  }
  int af() { return AF_INET; };
};

#ifdef AF_INET6
class InAddr6 : public InAddr {
  struct sockaddr_in6 s;
public:
  InAddr6(struct in6_addr *si) {
    s.sin6_family = AF_INET6;
    s.sin6_port = 0;
    s.sin6_addr = *si;
  }
  virtual ~InAddr6() { };
  void set_port(int port) { s.sin6_port = htons(port); };
  virtual string tostring() {
    char dest[100];
    inet_ntop(AF_INET6, &s.sin6_addr, dest, 100);
    dest[99] = 0;
    return dest;
  }
  sockaddr *addr() {
    return (sockaddr*)&s;
  }
  socklen_t addr_len() {
    return sizeof s;
  }
  int af() { return AF_INET6; };
};
#endif

InAddr *InAddr::resolv(const char *name) 
{
#if 0
#ifdef AF_INET6
  res_init();
  _res.options |= RES_USE_INET6;
#endif
#endif
  struct hostent *he = gethostbyname(name);
  if (!he)
    return NULL;

  if (he->h_addrtype == AF_INET)
    return new InAddr4((struct in_addr*)he->h_addr_list[0]);

#ifdef AF_INET6
  if (he->h_addrtype == AF_INET6)
    return new InAddr6((struct in6_addr*)he->h_addr_list[0]);
#endif

  return NULL;
}


Socket::Socket() {
  buffer = "";
  fd = -1;
}

int Socket::connect(InAddr *addr) {
  fd = socket(addr->af(), SOCK_STREAM, 0);
  if (fd == -1)
    return -1;
  
  int value = ::connect(fd, addr->addr(), addr->addr_len());
  
  int er = errno;
  
  if (value == -1) {
    ::close(fd);
    errno = er;
    return -1;
  }
  
  fcntl(fd, F_SETFL, O_NONBLOCK);
  dead = false;
  
  return true;
}

int Socket::read(char *data, int size) {
  if (dead) {
      errno = ECONNRESET;
      return -1;
  }
  int rval = ::read(fd, data, size);
  if (rval == -1) {
      if (errno == EAGAIN || errno == EWOULDBLOCK) {
	  return -1;
      }
      dead = true;
  }
  return rval;
}

int Socket::write(const char *data, int size) {
    if (dead) {
	errno = ECONNRESET;
	return -1;
    }
    int rval = ::write(fd, data, size);
    if (rval == -1) {
	dead = true;
    }
    return rval;
}

Socket::~Socket() {
  if (!dead) {
    shutdown(fd, 2);
    ::close(fd);
  }
}

void Socket::close() {
  shutdown(fd, 2);
  ::close(fd);
  dead = true;
}

